﻿using System;
using System.IO;
using System.Collections.Generic;

namespace PdpRtuDownloader
{
    /// <summary>
    /// Forwarding downloaded measurements into the web service.
    /// </summary>
    public class DataPump
    {
        private Configuration configuration;
        private Database.DomainObjects.RtuConfig rtuConfig;

        /// <summary>
        /// Gets common configuration.
        /// </summary>
        /// <value>The configuration.</value>
        public Configuration Configuration
        {
            get { return configuration; }
        }

        /// <summary>
        /// Gets the RTU station configuration.
        /// </summary>
        /// <value>The rtu config.</value>
        public Database.DomainObjects.RtuConfig RtuConfig
        {
            get { return rtuConfig; }
        }

        public DataPump(Configuration configuration, Database.DomainObjects.RtuConfig rtuConfig)
        {
            this.configuration = configuration;
            this.rtuConfig = rtuConfig;
        }

        /// <summary>
        /// Returns the directory, where measurement data files of a station are downloaded. Creates the directory if it does not exist.
        /// </summary>
        /// <returns>The station local directory.</returns>
        private string GetStationLocalDirectory()
        {
            // Get the directory where all downloaded files of the station are located.
            string stationLocalDirectory = Path.Combine(configuration.LocalDirectory, Convert.ToString(rtuConfig.StationId));

            // Create the directory if it does not exist.
            if (!Directory.Exists(stationLocalDirectory))
            {
                Directory.CreateDirectory(stationLocalDirectory);
            }

            return stationLocalDirectory;
        }


        public void Download()
        {
            try
            {
                // Get the directory where all downloaded files of the station are located.
                string stationLocalDirectory = GetStationLocalDirectory();

                // Prepare RTU station downloader.
                RtuStation.RtuStation station = new PdpRtuDownloader.RtuStation.RtuStation(rtuConfig);
                station.LocalDirectory = stationLocalDirectory;

                // Prepare web service connector.
                WebService.WebService webService = new WebService.WebService(configuration.WebServiceUri);

                // List all measurements.
                IEnumerable<string> allMeasurements = station.ListMeasurements();
                List<string> downloadMeasurements = new List<string>();

                // Loop through all measurements.
                foreach (string measurement in allMeasurements)
                {
                    // Do not download measurement if it already has been uploaded to the web service.
                    if (!webService.CheckMeasurementExists(measurement, rtuConfig.StationId))
                    {
                        // Do not download measurement if it already has been downloaded but unprocessed.
                        string localMeasurementFileName = Path.Combine(stationLocalDirectory, measurement + Constants.measurePostfix);
                        if (!File.Exists(localMeasurementFileName))
                        {
                            downloadMeasurements.Add(measurement);
                        }
                    }
                }

                // Download measurements.
                station.DownloadMeasurements(downloadMeasurements);
            }
            catch (Exception ex)
            {
                Console.WriteLine("{0}", ex.Message);
                Console.WriteLine("{0}", ex.ToString());
                // TODO
            }
        }

        public void Upload()
        {
            try
            {
                // Get the directory where all downloaded files of the station are located.
                string stationLocalDirectory = GetStationLocalDirectory();

                // Prepare web service connector.
                WebService.WebService webService = new WebService.WebService(configuration.WebServiceUri);

                // Returns all files in the specified directory.
                string[] fileNames = Directory.GetFiles(stationLocalDirectory);

                foreach (string fileName in fileNames)
                {
                    if (fileName.EndsWith(Constants.measurePostfix))
                    {
                        string fnWithoutDir = Path.GetFileName(fileName);
                        string measurement = fnWithoutDir.Substring(0, fnWithoutDir.Length - Constants.measurePostfix.Length);
                        Measurement.Loader loader = new PdpRtuDownloader.Measurement.Loader();
                        Measurement.MeasurementData measurementData = loader.LoadMeasurementData(measurement, stationLocalDirectory);

                        // Upload measured date into the web service.
                        webService.UploadMeasurementData(measurementData, rtuConfig.StationId);

                        // Delete local measurement files.
                        string archiveInputFileName = Path.Combine(stationLocalDirectory, measurement + Constants.measurePostfix);
                        string peakClassInputFileName = Path.Combine(stationLocalDirectory, measurement + Constants.peakClassDataPostfix);

                        File.Delete(archiveInputFileName);
                        File.Delete(peakClassInputFileName);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("{0}", ex.Message);
                Console.WriteLine("{0}", ex.ToString());
                // TODO
            }
        }
    }
}

